class Spot < ActiveRecord::Base
  
  #同じ経度で直線距離1kmとなる緯度の差
  LATI_1KM =   0.009014
  #同じ緯度で直線距離1kmとなる緯度の差
  LONGI_1KM = 0.0110095
  # 画像のパス
  UPLOAD_IMAGE_DIR_PATH = File.join(Rails.root, "files/spots")
  
  # 斐川観光協会dir_name
  HIKAWA_DIR_NAME = "hikawano"

  belongs_to :organization

  has_many :spot_categories, :dependent => :destroy, :validate => false
  accepts_nested_attributes_for :spot_categories, :allow_destroy => true

  has_many :spot_divisions, :dependent => :destroy, :validate => false
  accepts_nested_attributes_for :spot_divisions, :allow_destroy => true

  has_one :coupon, :dependent => :destroy
  
  # -- validates
  validates :name, :presence => true
  validates :organization_id, :presence => true
  
  # 引数の位置情報から、周辺のスポットを絞り込む 5km
  scope :near_spot, ->(lat, lon) do
    where("latitude <= ? AND latitude >= ? AND longitude <= ? AND longitude >= ?", 
      (lat.to_f + LATI_1KM * 5), (lat.to_f - LATI_1KM * 5), (lon.to_f + LONGI_1KM * 5), (lon.to_f - LONGI_1KM * 5))
  end
  
  scope :hikawa, joins(:organization).where("organizations.dir_name = ?", HIKAWA_DIR_NAME)
  
  # スポットの画像が保存されているディレクトリの絶対パスを返す。
  def image_dir_path
    if self.new_record? || self.image.blank?
      return ""
    end
    File.join(UPLOAD_IMAGE_DIR_PATH, self.id.to_s)
  end
  
  # スポットの画像の絶対パスを返す。
  def image_path
    if self.new_record? || self.image.blank?
      return ""
    end
    File.join(self.image_dir_path, self.image)
  end
  
  #   spot.distance(latitude, longitude, boolean) -> float
  #     
  # ヒュベニの計算式により緯度・経度から、代入された座標との直線距離を返す
  # 
  #   spot = Spot.new(latitude: 35.395672, longitude: 132.869297)
  #   spot.distance(35.405683, 132.869298)
  #   #=> 1.1
  #   spot.distance(35.405683, 132.869298, true)
  #   #=> 1.11
  def distance(lat,longi,accuracy=false)
    dp = (self.latitude - lat).abs
    dr = (self.longitude - longi).abs
    p = ((self.latitude + lat) / 2)*Math::PI/180
    t = (1-0.006674*Math.sin(p)*Math.sin(p))
    m = 6334834 / (Math::sqrt(t*t*t))
    n = 6377397 / (Math::sqrt(t))
    # accuracy が true のとき、小数点以下2桁目まで出力する
    if accuracy
      div_nums = [10,100.0]
    else
      div_nums = [100,10.0]
    end
    return ((Math::sqrt((m*dp*Math::PI/180)*(m*dp*Math::PI/180)+(n*Math.cos(p)*dr*Math::PI/180)*(n*Math.cos(p)*dr*Math::PI/180)))/div_nums[0]).round/div_nums[1]
  end

  #   spot.near_range?(latitude, logintude) -> true or false
  #
  # スポットに対して、経度、緯度を渡し、渡した値がスポットに近いかを返す。<br />
  # 近いの基準は縦、横それぞれ5km以内かどうかで判断をする。
  #
  #   spot = Spot.new(latitude: 0.001, longitude: 0.001)
  #   spot.near_all_range?(0.001 + LATI_1KM, 0.001 + LONGI_1KM)
  #   #=> true
  #   spot.near_all_range?(0.001 + (LATI_1KM*6), 0.001 + (LONGI_1KM*6))
  #   #=> false
  def near_range?(lat, longi)
    max_lat =   lat    + LATI_1KM    * 5 # +5km
    min_lat =   lat    - LATI_1KM    * 5 # -5km
    max_longi = longi  + LONGI_1KM  * 5 # +5km
    min_longi = longi  - LONGI_1KM  * 5 # -5km
    return (self.latitude <= max_lat && self.latitude >= min_lat) && (self.longitude <= max_longi && self.longitude >= min_longi)
  end

  #   spot.near_all_range?(latitude, logintude) -> true or false
  #
  # スポットに対して、経度、緯度を渡し、渡した値がスポットに近いかを返す。<br />
  # 近いの基準は縦、横それぞれ2km以内かどうかで判断をする。
  #
  #   spot = Spot.new(latitude: 0.001, longitude: 0.001)
  #   spot.near_all_range?(0.001 + LATI_1KM, 0.001 + LONGI_1KM)
  #   #=> true
  #   spot.near_all_range?(0.001 + (LATI_1KM*3), 0.001 + (LONGI_1KM*3))
  #   #=> false
  def near_all_range?(lat, longi)
    max_lat =   lat    + LATI_1KM    * 2 # +2km
    min_lat =   lat    - LATI_1KM    * 2 # -2km
    max_longi = longi  + LONGI_1KM  * 2 # +2km
    min_longi = longi  - LONGI_1KM  * 2 # -2km
    return (self.latitude <= max_lat && self.latitude >= min_lat) && (self.longitude <= max_longi && self.longitude >= min_longi)
  end

  # 現在未使用
  # 
  def yorimichi_range?(start_lat, start_longi, end_lat, end_longi)
    if start_lat >= end_lat
      max_lat = start_lat
      min_lat = end_lat
    else
      max_lat = end_lat
      min_lat = start_lat
    end
    if start_longi >= end_longi
      max_longi = start_longi
      min_longi = end_longi
    else
      max_longi = end_longi
      min_longi = start_longi
    end
    organization = self.organization
    max_lat += organization.trip_height
    min_lat -= organization.trip_height
    max_longi += organization.trip_width
    min_longi -= organization.trip_width
    return (self.latitude <= max_lat && self.latitude >= min_lat) && (self.longitude <= max_longi && self.longitude >= min_longi)
  end
  
end
